/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.service.registry;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.tinygroup.service.Service;

/**
 * 服务注册表，提供对服务注册表的各项操作 每种执行器都有自己的服务注册表
 * 
 * @author luoguo
 * 
 */
public interface ServiceRegistry {
	String BEAN_NAME = "serviceRegistry";

	/**
	 * 注册服务，重复注册时旧的被替换
	 * 
	 * @param service
	 */
	void registeService(ServiceRegistryItem serviceRegistryItem);

	/**
	 * 返回指定服务的服务注册信息
	 * 
	 * @param service
	 * @return
	 */
	ServiceRegistryItem getServiceRegistryItem(Service service);

	/**
	 * 注册一批服务，重复注册时旧的被替换
	 * 
	 * @param serviceRegistryItems
	 */
	void registeService(List<ServiceRegistryItem> serviceRegistryItems);

	/**
	 * 注册一批服务，重复注册时旧的被替换
	 * 
	 * @param serviceRegistryItems
	 */

	void registeService(ServiceRegistryItem[] serviceRegistryItems);

	/**
	 * 注册一批服务，重复注册时旧的被替换
	 * 
	 * @param serviceRegistryItems
	 */
	void registeService(Set<ServiceRegistryItem> serviceRegistryItems);

	/**
	 * 删除服务
	 * 
	 * @param serviceId
	 */
	void removeService(String serviceId, String version);

	/**
	 * 
	 * @param serviceId
	 * @param version
	 */
	void removeService(String groupId, String artifactId, String name,
			String version);

	/**
	 * 返回注册服务个数
	 * 
	 * @return
	 */
	int size();

	/**
	 * 清空所有服务
	 */
	void clear();


	/**
	 * 查找指定服务对应的注册项，没有指定版本时，采用最新的版本
	 * 
	 * @param serviceId
	 * @return @
	 */
	ServiceRegistryItem getServiceRegistryItem(String serviceId);

	/**
	 * 查找指定服务对应的注册项
	 * 
	 * @param serviceId
	 * @param version
	 * @return @
	 */
	ServiceRegistryItem getServiceRegistryItem(String serviceId, String version);

	/**
	 * 查找指定服务对应的注册项，没有指定版本时，采用最新的版本
	 * 
	 * @param groupId
	 * @param artifactId
	 * @param name
	 * @return @
	 */
	ServiceRegistryItem getServiceRegistryItem(String groupId,
			String artifactId, String name);

	/**
	 * 查找指定服务对应的注册项
	 * 
	 * @param groupId
	 * @param artifactId
	 * @param name
	 * @param version
	 * @return @
	 */
	ServiceRegistryItem getServiceRegistryItem(String groupId,
			String artifactId, String name, String version);

	/**
	 * 获取所有服务注册项
	 * 
	 * @return 所有的服务注册项
	 */
	Collection<ServiceRegistryItem> getServiceRegistryItems();
}
